// Author: tataylino.com
#include <SPI.h>
#include <Adafruit_GFX.h>
#include <TFT_ILI9163C.h>
#include <ESP8266WiFi.h>
#include <Wire.h>
 
const char* ssid = "MyWifi"; // your Wifi name
const char* password = "password123"; // Wifi Password
 
WiFiServer server(80);

// TDA7419 controls
#define VOLUME_ADDR   3
#define SOURCE_ADDR   0
#define BASS_ADDR     6
#define MID_ADDR      5
#define TREBLE_ADDR   4
#define SACLK         D0  
   
// Color definitions
#define  BLACK   0x0000
#define BLUE    0x001F
#define RED     0xF800
#define GREEN   0x07E0
#define CYAN    0x07FF
#define MAGENTA 0xF81F
#define YELLOW  0xFFE0  
#define WHITE   0xFFFF

#define  NBINS                7
const uint8_t bar_Width =     3;
uint32_t     avrg_TmrF =      0;
uint16_t     t_b[NBINS];
uint16_t datax_[NBINS];
uint16_t dataraw_[NBINS];
String src = "TV";
const int SAOUT = A0; //analog input pin

#define CS D4
#define DC D6
#define RST D8

TFT_ILI9163C tft = TFT_ILI9163C(CS, RST, DC);
// LCD Connection (3v,gnd, d4, d6, d8, d7, gnd, 3v )
uint8_t vol = 10;
uint8_t bass = 11;
uint8_t mid = 12;
uint8_t tre = 13;

void setup(void) {
  pinMode(SACLK, OUTPUT);
  WiFi.begin(ssid, password);
  Wire.begin(D1, D2); /* join i2c bus with SDA=D1 and SCL=D2 of NodeMCU */
  Wire.setClock(10000L);
  tft.begin();
  tft.setRotation(1);
  tft.fillScreen(BLACK);
  tft.setTextWrap(true);
  tft.setTextColor(WHITE, BLACK);
  tft.setCursor(0,50);
  tft.println("Connecting to WiFi");
  while (WiFi.status() != WL_CONNECTED) {
    delay(100);
    //Serial.print(".");
    tft.println(".");
  }
  tft.fillScreen(BLACK);
  // Start the server
  server.begin();
  // Print the IP address
  tft.setCursor(40,110);
  tft.println(WiFi.localIP());
  display_volume_level();

  // initialize TDA7419
  vol = 60;
  bass = 10;
  mid = 0;
  tre = 10;
    //reset system - write 0 to all registers
  for (int i=0;i<18;i++){
    set_tda7419(i,0);
    delay(50);
  }
  set_tda7419(VOLUME_ADDR,80-vol); //3
  delay(50);
  set_tda7419(SOURCE_ADDR,4); // 0
  delay(50);
  set_tda7419(BASS_ADDR,bass);
  delay(50);
  set_tda7419(MID_ADDR,mid);
  delay(50);
  set_tda7419(TREBLE_ADDR,tre);
  delay(50);
  set_tda7419(16,214); //(16,22); // spectrum analyzer reset off
  delay(50);
  set_tda7419(1,0); // main loudness
  delay(50);
  set_tda7419(2,2); // soft mute - i2c mute only
  delay(50);
  set_tda7419(10,15); // Speaker attenuator LF
  delay(50);
  set_tda7419(11,15); // Speaker attenuator RF
  delay(50);
  set_tda7419(12,15); // Speaker attenuator LR
  delay(50);
  set_tda7419(13,15); // Speaker attenuator RR
  delay(50);
  set_tda7419(4,128); // treble filter
  debug_display();
}

//----------------   MAIN   -----------------//
void loop(){
  read_spectrum(); // read spectrum analyzer output
  //dataraw_[0] = 12;
  for (int i=0;i<NBINS;i++){
    //datax_[i] = random(0,64);//random(0,1024);
    //datax_[i] = map(dataraw_[i],0,1024,0,64);
    datax_[i] = constrain(dataraw_[i],0,64);
    
  }
  //Print_Data();
  spectrumdisplay(datax_);
  delay(50);
   // Check if a client has connected
  WiFiClient client = server.available();
  if (!client) {
    return;
  }

  // Read the first line of the request
  String request = client.readStringUntil('\r');
  Serial.println(request);
  client.flush();
 
  // Match the request
  //int value = LOW;
  if (request.indexOf("/VOL=UP") != -1)  {
    vol++;
    vol = constrain(vol, 0, 80); // limit value range
    set_tda7419(VOLUME_ADDR,80-vol);
    display_volume_level();
  }
  if (request.indexOf("/VOL=DOWN") != -1)  {
    vol--;
    vol = constrain(vol, 0, 80); // limit value range
    set_tda7419(VOLUME_ADDR,80-vol);
    display_volume_level();
  }
 if (request.indexOf("/SRC=TV") != -1)  {
    src = "TV";
    set_tda7419(SOURCE_ADDR,0);
    display_volume_level();
 }
 if (request.indexOf("/SRC=DVD") != -1)  {
    src = "DVD";
    set_tda7419(SOURCE_ADDR,1);
    display_volume_level();
 }
  if (request.indexOf("/SRC=BLUETOOTH") != -1)  {
  src = "BLUETOOTH";
  set_tda7419(SOURCE_ADDR,4);
  display_volume_level();
 }
  if (request.indexOf("/BASS=DOWN") != -1)  {
    bass--;
    bass = constrain(bass, 0, 31); // limit value range
    set_tda7419(BASS_ADDR,bass);
    display_volume_level();
  }
  if (request.indexOf("/BASS=UP") != -1)  {
    bass++;
    bass = constrain(bass, 0, 31); // limit value range
    set_tda7419(BASS_ADDR,bass);
    display_volume_level();
  }
  if (request.indexOf("/MID=DOWN") != -1)  {
    mid--;
    mid = constrain(mid, 0, 31); // limit value range
    set_tda7419(MID_ADDR,mid);
    display_volume_level();
  }
  if (request.indexOf("/MID=UP") != -1)  {
    mid++;
    mid = constrain(mid, 0, 31); // limit value range
    set_tda7419(MID_ADDR,mid);
    display_volume_level();
  }
  if (request.indexOf("/TRE=DOWN") != -1)  {
    tre--;
    tre = constrain(tre, 0, 31); // limit value range
    set_tda7419(TREBLE_ADDR,tre);
    display_volume_level();
  }
  if (request.indexOf("/TRE=UP") != -1)  {
    tre++;
    tre = constrain(tre, 0, 31); // limit value range
    set_tda7419(TREBLE_ADDR,tre);
    display_volume_level();
  }

  // Return the response
  client.println("HTTP/1.1 200 OK");
  client.println("Content-Type: text/html");
  client.println(""); //  do not forget this one
  client.println("<!DOCTYPE HTML>");
  client.println("<html>");
  client.println("<head><title>Tataylino's Home</title></head>");
  client.print("<body bgcolor = grey>");
  //client.print("<hr><h2>Tataylino's TV </h2><br>");
  client.print("<hr> ");
  client.print("<h2>Tataylino's Radio </h2><br>");
  client.print("Volume Level: ");
  client.print(vol);
  client.println("<br><br>");
  client.println("<a href=\"/VOL=DOWN\"\"><button> VOLUME - </button></a>   ");  
  client.println("<a href=\"/VOL=UP\"\"><button> VOLUME + </button></a>");

  client.println("<br><br>Source: ");
  client.print(src);
  client.println("<br><br>");
  client.println("<a href=\"/SRC=TV\"\"><button> TV </button></a>   ");  
  client.println("<a href=\"/SRC=BLUETOOTH\"\"><button> BLUETOOTH </button></a>   ");
  client.println("<a href=\"/SRC=DVD\"\"><button> DVD </button></a>");

  client.print("<br><br>Bass Level: ");
  client.print(bass);
  client.println("<br><br>");
  client.println("<a href=\"/BASS=DOWN\"\"><button> Bass - </button></a>   ");  
  client.println("<a href=\"/BASS=UP\"\"><button> Bass + </button></a>");

  client.print("<br><br>Mid Level: ");
  client.print(mid);
  client.println("<br><br>");
  client.println("<a href=\"/MID=DOWN\"\"><button> Mid - </button></a>   ");  
  client.println("<a href=\"/MID=UP\"\"><button> Mid + </button></a>");

  client.print("<br><br>Treble Level: ");
  client.print(tre);
  client.println("<br><br>");
  client.println("<a href=\"/TRE=DOWN\"\"><button> Treble - </button></a>   ");  
  client.println("<a href=\"/TRE=UP\"\"><button> Treble + </button></a>");

  client.println("</body></html>");

}
//-------------------------------------------//
void debug_display()
{
  tft.fillRect(10,90,100,10,BLACK); //fillRect(X,Y,width,height,color)
  tft.setCursor(10, 90);
  tft.println("tataylino.com"); //(dataraw_[0]);
  //tft.println(c);
}
void read_spectrum()
{//  Read Spectrum Analyzer Output
  digitalWrite(SACLK, HIGH);
  delay(55); // Reset
  digitalWrite(SACLK, LOW);
  delay(2);
  for (int i=0;i<NBINS;i++){
    digitalWrite(SACLK, HIGH);
    delay(2);
    digitalWrite(SACLK, LOW);
    delay(1);
    dataraw_[i] = analogRead(SAOUT);
    delay(1);
  }
}

void set_tda7419(uint8_t param, uint8_t val)
{
  Wire.beginTransmission(68);//(136); /* begin with device address 0x88 */
  Wire.write(param);            // sends instruction byte
  Wire.write(val);             // sends potentiometer value byte
  Wire.endTransmission();    /* stop transmitting */
}

void display_volume_level()
{
  tft.setCursor(2, 0);
  tft.setTextColor(GREEN);
  tft.setTextSize(1);
  tft.println("Vol:");
  tft.fillRect(25,0,100,10,BLACK);
  tft.setCursor(25, 0);
  tft.println(vol);
  // source
  tft.setCursor(10, 100);
  tft.setTextColor(GREEN);
  tft.setTextSize(1);
  tft.println("Input:");
  tft.fillRect(50,100,60,10,BLACK);
  tft.setCursor(50, 100);
  tft.println(src); 
  // Bass
  tft.setCursor(45, 0);
  tft.println("B:");
  tft.setCursor(55, 0);
  tft.println(bass);
  // Mid
  tft.setCursor(72, 0);
  tft.println("M:");
  tft.setCursor(82, 0);
  tft.println(mid);
  // Tre
  tft.setCursor(100, 0);
  tft.println("T:");
  tft.setCursor(110, 0);
  tft.println(tre);
}

void spectrumdisplay(uint16_t datax[])
{
  uint8_t bar_width = 5;
  uint8_t bar_maxheight = 64; // max height
  uint8_t bar_x = 10;
  uint8_t bar_y = 15;
  uint8_t numbars = 7;
  uint8_t bar_spacing = 11;
  uint8_t bar_xpos;
  uint8_t bar_ypos;
  static uint8_t debug = 10;

  // display bars
  for(uint8_t i = 0; i <= numbars-1; i++)
  {
    // bars
    bar_xpos = bar_x + (i * (bar_width + bar_spacing));
    tft.fillRect(bar_xpos, bar_y + bar_maxheight - datax[i] + 1, bar_width, datax[i] + 1, CYAN); //fillRect(X,Y,width,height,color)
    tft.fillRect(bar_xpos, bar_y, bar_width, bar_maxheight - datax[i], BLACK); //fillRect(X,Y,width,height,color)   <- Black fill
  }
  // text
  tft.setCursor(10, 82);
  tft.setTextColor(GREEN);
  tft.setTextSize(1);
  tft.println("60 400  1k  6k 16k");
  
  tft.setCursor(10, 110);
  tft.setTextColor(GREEN);
  tft.setTextSize(1);
  tft.println("IP:");
}

